import logging
import logging.handlers
import os
import warnings

class Singleton(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

class Logger(metaclass=Singleton):
    def __init__(self):
        level = os.environ.get('FASTNET_LOG_LEVEL', 'INFO')
        logLevel = getattr(logging, level)
        warnings.filterwarnings('default')
        logging.captureWarnings(True)
        self.logger = logging.getLogger('py.warnings')
        self.logger.setLevel(logLevel)

        stderr_log_handler = logging.StreamHandler()
        stderr_log_handler.setLevel(logLevel)
        stderr_log_handler.setFormatter(logging.Formatter('%(module)s: %(levelname)s - %(message)s'))
        self.logger.addHandler(stderr_log_handler)


logger = Logger().logger




import collections.abc

class ObjectView(dict):
    """Data structure that improves a regular map structure so
    its components can be accessed as map['value'] or map.value.
    It works only for string keys.
    """
    def __getattr__(self, name):
        if name in self: return self[name]
        else: raise AttributeError("No such attribute: " + name)

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        if name in self: del self[name]
        else: raise AttributeError("No such attribute: " + name)


def to_list(val):
    """
    If val is an iterable collection other than a string, returns val,
    otherwise, returs [val].
    """
    if isinstance(val, collections.abc.Iterable) and type(val) is not str:
        return val
    else: return [val]


def smart_tuple(vec):
    """
    If len(vec) == 1, returns vec[0], otherwise, returns vec as a tuple.
    """
    if len(vec) == 1: return vec[0]
    else: return tuple(vec)



import os
import numpy
import re

npType = numpy.float32

#PYTORCH_DEVICE can be: cpu, gpu, gpu:0, gpu:1, etc...
filt = re.compile('([a-z]+):?(\d?)')
grp = filt.match(os.environ.get('PYTORCH_DEVICE', 'cpu'))
device = grp.group(1)
id =  int(grp.group(2)) if grp.group(2) != '' else 0

def set_torch_defaults(torch):
    if device == 'gpu':
        torch.cuda.set_device(id)
        torch.set_default_tensor_type(torch.cuda.FloatTensor)
    else:
        torch.set_default_tensor_type(torch.FloatTensor)

    torch.set_default_dtype(torch.float32)
